/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";

@Component({
  selector: 'app-editor',
  templateUrl: './editor.component.html',
  styleUrls: ['./editor.component.scss']
})
export class EditorComponent implements OnInit {
  private _textToEdit: string;

  @Input() buttonText = "";
  @Input() customStyle: boolean = false
  @Input() shouldDisable: boolean = false
  // Set original data only when we're ready
  @Input() set textToEdit(data: string) {
    this._textToEdit = data;
    this.originalData = data;
  }
  @Input() modalTitleText: string;
  @Output() newEditEvent = new EventEmitter<string>();
  @Output() editorOpenEvent = new EventEmitter<boolean>();

  public originalData: string;
  public editedData: string;
  private isEditorOpen: boolean = false;

  constructor(
    private modalService: NgbModal,
  ) { }

  public open(content) {
    this.toggleEditorOpen()
    this.modalService.open(content, { ariaLabelledBy: "modal-title", centered: true, size: "lg" }).result.then(
      (result) => {
        if (result === "save-and-close" && this.editedData) {
          // "Save" button clicked; emit edited data to parent component
          this.newEditEvent.emit(this.editedData)
          this.toggleEditorOpen()
        } else {
          // Form was exited by clicking out of it or pressing ESC
          this.resetForm();
          this.toggleEditorOpen()
        }
      },
      (reason) => {
        // Form was exited using Cancel button or X
        this.resetForm();
        this.toggleEditorOpen()
      },
    );
  }

  private toggleEditorOpen() {
    this.isEditorOpen = !this.isEditorOpen
    this.editorOpenEvent.emit(this.isEditorOpen)
  }

  private resetForm() {
    this.undoEditChanges()
  }

  public undoEditChanges() {;
    this._textToEdit = this.originalData
    this.editedData = this._textToEdit
  }

  public edit(changes: string) {
    this.editedData = changes
    this._textToEdit = this.editedData
  }

  ngOnInit(): void {}

}
