"""run quicklook in pims_split

Revision ID: c635a4694c6d
Revises: 8a6aa0fd4a91
Create Date: 2023-02-24 16:15:19.758945

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'c635a4694c6d'
down_revision = '8a6aa0fd4a91'
branch_labels = None
depends_on = None

# Adding new split layout to the metadata.json file
old_metadata = """{"systemId": "{{request_id}}", "fileSetIds": ["{{sdmId}}", "{{calSdmId}}"], "creationTime": "{{created_at}}", "workflowName": "pims_split",  "productLocator": "{{product_locator}}", "calProductLocator": "{{cal_locator}}", "destinationDirectory": "{{root_directory}}/{{relative_path}}", "token": "{{token}}", "splits": "{{#splits}}{{.}} {{/splits}}", "casaHome": "{{casaHome}}", "data_location": "{{data_location}}", "vlass_product": "{{vlass_product}}", "existing_restore": "{{existing_restore}}", "need_project_metadata": "{{need_project_metadata}}"}"""

new_metadata = """{"systemId": "{{request_id}}", "fileSetIds": ["{{sdmId}}", "{{calSdmId}}"], "creationTime": "{{created_at}}", "workflowName": "pims_split",  "productLocator": "{{product_locator}}", "calProductLocator": "{{cal_locator}}", "destinationDirectory": "{{root_directory}}/{{relative_path}}", "token": "{{token}}", "splits": ["{{split_dir}}", "{{quicklook_url}}"], "casaHome": "{{casaHome}}", "data_location": "{{data_location}}", "vlass_product": "{{vlass_product}}", "existing_restore": "{{existing_restore}}", "need_project_metadata": "{{need_project_metadata}}"}"""

# Using new split_dir in the DAG file and setting quicklook_url as an argument if it exists
old_dag = """JOB RESTORE restore.condor
VARS RESTORE jobname="$(JOB)"

{{#splits}}
JOB {{#make_condor_jobname}}{{.}}{{/make_condor_jobname}} split.condor
VARS {{#make_condor_jobname}}{{.}}{{/make_condor_jobname}} jobname="$(JOB)" split_dir="{{.}}"

{{/splits}}

JOB FINISH write_finished_file.condor
VARS FINISH jobname="$(JOB)"

PARENT RESTORE CHILD {{#splits}}{{#make_condor_jobname}}{{.}}{{/make_condor_jobname}} {{/splits}}
PARENT {{#splits}}{{#make_condor_jobname}}{{.}}{{/make_condor_jobname}} {{/splits}} CHILD FINISH
"""

new_dag = """JOB RESTORE restore.condor
VARS RESTORE jobname="$(JOB)"

{{#splits}}
JOB {{#make_condor_jobname}}{{split_dir}}{{/make_condor_jobname}} split.condor
VARS {{#make_condor_jobname}}{{split_dir}}{{/make_condor_jobname}} jobname="$(JOB)" split_dir="{{split_dir}}" {{#quicklook_url}}quicklook_url="{{quicklook_url}}"{{/quicklook_url}}

{{/splits}}

JOB FINISH write_finished_file.condor
VARS FINISH jobname="$(JOB)"

PARENT RESTORE CHILD {{#splits}}{{#make_condor_jobname}}{{split_dir}}{{/make_condor_jobname}} {{/splits}}
PARENT {{#splits}}{{#make_condor_jobname}}{{split_dir}}{{/make_condor_jobname}} {{/splits}} CHILD FINISH
"""

# Add quicklook_url as the second argument to split.sh (will print empty if it doesn't exist in the json)
old_condor_args = 'arguments = "$(split_dir)"'

new_condor_args = 'arguments = "$(split_dir)" "$(quicklook_url)"'

# Calling quicklook after pimscache in the split job
old_split = """#!/bin/sh
set -o errexit
export HOME=$TMPDIR
TILE=$(echo $1 | cut -d "/" -f 1)
PHCENTER=$(echo $1 | cut -d "/" -f 2)

# Get the measurement set path
{{^existing_restore}}
MS={{data_location}}/working/*.ms
{{/existing_restore}}
{{#existing_restore}}
MS={{existing_restore}}
{{/existing_restore}}

# Link it in the splits rawdata
ln -s $MS rawdata/

# Run CASA
./casa_envoy --split metadata.json PPR.xml

# Populate cache
./pimscache cp -c {{vlass_product}} -t $TILE -p $PHCENTER working/*_split.ms
"""

new_split = """#!/bin/sh
set -o errexit
export HOME=$TMPDIR
TILE=$(echo $1 | cut -d "/" -f 1)
PHCENTER=$(echo $1 | cut -d "/" -f 2)

# Get the measurement set path
{{^existing_restore}}
MS={{data_location}}/working/*.ms
{{/existing_restore}}
{{#existing_restore}}
MS={{existing_restore}}
{{/existing_restore}}

# Link it in the splits rawdata
ln -s $MS rawdata/

# Run CASA
./casa_envoy --split metadata.json PPR.xml

# Populate cache
./pimscache cp -c {{vlass_product}} -t $TILE -p $PHCENTER working/*_split.ms

# Run quicklook if second parameter was given
if ! [[ -z "$2" ]]; then
    curl --request PUT --header "Content-Length: 0" $2
fi
"""

def upgrade():
    op.execute(f"""
        UPDATE workflow_templates
        SET content = E'{new_metadata}'
        WHERE workflow_name = 'pims_split' AND filename = 'metadata.json' 
    """)

    op.execute(f"""
        UPDATE workflow_templates
        SET content = E'{new_dag}'
        WHERE workflow_name = 'pims_split' AND filename = 'pims_split.dag' 
    """)

    op.execute(f"""
        UPDATE workflow_templates
        SET content = replace(convert_from(content, 'utf8'), E'{old_condor_args}', E'{new_condor_args}')::bytea
        WHERE workflow_name = 'pims_split' AND filename = 'split.condor' 
    """)

    op.execute(f"""
        UPDATE workflow_templates
        SET content = E'{new_split}'
        WHERE workflow_name = 'pims_split' AND filename = 'split.sh' 
    """)


def downgrade():
    op.execute(f"""
        UPDATE workflow_templates
        SET content = E'{old_metadata}'
        WHERE workflow_name = 'pims_split' AND filename = 'metadata.json' 
    """)

    op.execute(f"""
            UPDATE workflow_templates
            SET content = E'{old_dag}'
            WHERE workflow_name = 'pims_split' AND filename = 'pims_split.dag' 
        """)

    op.execute(f"""
            UPDATE workflow_templates
            SET content = replace(convert_from(content, 'utf8'), E'{new_condor_args}', E'{old_condor_args}')::bytea
            WHERE workflow_name = 'pims_split' AND filename = 'split.condor' 
        """)

    op.execute(f"""
            UPDATE workflow_templates
            SET content = E'{old_split}'
            WHERE workflow_name = 'pims_split' AND filename = 'split.sh' 
        """)
